@@ -23,9 +23,11 @@ from collections import OrderedDict
 from tools import building
 from tools import diagnostics
 from tools import shared
+from tools import utils
 from tools import gen_struct_info
 from tools import webassembly
-from tools.shared import WINDOWS, path_from_root, exit_with_error, asmjs_mangle
+from tools.utils import exit_with_error, path_from_root
+from tools.shared import WINDOWS, asmjs_mangle
 from tools.shared import treat_as_user_function, strip_prefix
 from tools.settings import settings
 
@@ -77,13 +79,13 @@ def optimize_syscalls(declares, DEBUG):
     # without filesystem support, it doesn't matter what syscalls need
     settings.SYSCALLS_REQUIRE_FILESYSTEM = 0
   else:
-    syscall_prefixes = ('__sys', 'fd_')
+    syscall_prefixes = ('__syscall_', 'fd_')
     syscalls = [d for d in declares if d.startswith(syscall_prefixes)]
     # check if the only filesystem syscalls are in: close, ioctl, llseek, write
     # (without open, etc.. nothing substantial can be done, so we can disable
     # extra filesystem support in that case)
     if set(syscalls).issubset(set([
-      '__sys_ioctl',
+      '__syscall_ioctl',
       'fd_seek',
       'fd_write',
       'fd_close',
@@ -112,11 +114,11 @@ def update_settings_glue(metadata, DEBUG):
   if settings.SIDE_MODULE:
     # we don't need any JS library contents in side modules
     settings.DEFAULT_LIBRARY_FUNCS_TO_INCLUDE = []
-
-  all_funcs = settings.DEFAULT_LIBRARY_FUNCS_TO_INCLUDE + [shared.JS.to_nice_ident(d) for d in metadata['declares']]
-  settings.DEFAULT_LIBRARY_FUNCS_TO_INCLUDE = sorted(set(all_funcs).difference(metadata['exports']))
-
-  settings.DEFAULT_LIBRARY_FUNCS_TO_INCLUDE += metadata['globalImports']
+  else:
+    syms = settings.DEFAULT_LIBRARY_FUNCS_TO_INCLUDE + [shared.JS.to_nice_ident(d) for d in metadata['declares']]
+    syms = set(syms).difference(metadata['exports'])
+    syms.update(metadata['globalImports'])
+    settings.DEFAULT_LIBRARY_FUNCS_TO_INCLUDE = sorted(syms)
 
   settings.WASM_EXPORTS = metadata['exports'] + list(metadata['namedGlobals'].keys())
   # Store function exports so that Closure and metadce can track these even in
@@ -154,7 +156,7 @@ def apply_static_code_hooks(forwarded_json, code):
   code = shared.do_replace(code, '<<< ATINITS >>>', str(forwarded_json['ATINITS']))
   if settings.HAS_MAIN:
     code = shared.do_replace(code, '<<< ATMAINS >>>', str(forwarded_json['ATMAINS']))
-  if settings.EXIT_RUNTIME:
+  if settings.EXIT_RUNTIME and (not settings.MINIMAL_RUNTIME or settings.HAS_MAIN):
     code = shared.do_replace(code, '<<< ATEXITS >>>', str(forwarded_json['ATEXITS']))
   return code
 
@@ -174,7 +176,7 @@ def compile_settings():
     # Call js compiler
     env = os.environ.copy()
     env['EMCC_BUILD_DIR'] = os.getcwd()
-    out = shared.run_js_tool(path_from_root('src', 'compiler.js'),
+    out = shared.run_js_tool(path_from_root('src/compiler.js'),
                              [settings_file], stdout=subprocess.PIPE, stderr=stderr_file,
                              cwd=path_from_root('src'), env=env)
   assert '//FORWARDED_DATA:' in out, 'Did not receive forwarded data in pre output - process failed?'
@@ -371,13 +373,11 @@ def emscript(in_wasm, out_wasm, outfile_js, memfile, DEBUG):
 
 
 def remove_trailing_zeros(memfile):
-  with open(memfile, 'rb') as f:
-    mem_data = f.read()
+  mem_data = utils.read_binary(memfile)
   end = len(mem_data)
   while end > 0 and (mem_data[end - 1] == b'\0' or mem_data[end - 1] == 0):
     end -= 1
-  with open(memfile, 'wb') as f:
-    f.write(mem_data[:end])
+  utils.write_binary(memfile, mem_data[:end])
 
 
 def finalize_wasm(infile, outfile, memfile, DEBUG):
@@ -398,7 +398,7 @@ def finalize_wasm(infile, outfile, memfile, DEBUG):
     building.save_intermediate(infile + '.map', 'base_wasm.map')
     args += ['--output-source-map-url=' + settings.SOURCE_MAP_BASE + os.path.basename(outfile) + '.map']
     modify_wasm = True
-  if settings.DEBUG_LEVEL >= 2 or settings.ASYNCIFY_ADD or settings.ASYNCIFY_ADVISE or settings.ASYNCIFY_ONLY or settings.ASYNCIFY_REMOVE or settings.EMIT_SYMBOL_MAP or settings.PROFILING_FUNCS:
+  if settings.DEBUG_LEVEL >= 2 or settings.ASYNCIFY_ADD or settings.ASYNCIFY_ADVISE or settings.ASYNCIFY_ONLY or settings.ASYNCIFY_REMOVE or settings.EMIT_SYMBOL_MAP or settings.EMIT_NAME_SECTION:
     args.append('-g')
   if settings.WASM_BIGINT:
     args.append('--bigint')
@@ -509,6 +509,10 @@ def add_standard_wasm_imports(send_items_map):
 
   if settings.RELOCATABLE:
     send_items_map['__indirect_function_table'] = 'wasmTable'
+    if settings.EXCEPTION_HANDLING:
+      send_items_map['__cpp_exception'] = '___cpp_exception'
+    if settings.SUPPORT_LONGJMP == 'wasm':
+      send_items_map['__c_longjmp'] = '___c_longjmp'
 
   if settings.MAYBE_WASM2JS or settings.AUTODEBUG or settings.LINKABLE:
     # legalization of i64 support code may require these in some modes
@@ -778,10 +782,6 @@ def load_metadata_wasm(metadata_raw, DEBUG):
       exit_with_error('unexpected metadata key received from wasm-emscripten-finalize: %s', key)
     metadata[key] = value
 
-  # TODO(sbc): Remove this once binaryen change to globalImports has been around for a while.
-  if 'externs' in metadata_json:
-    metadata['globalImports'] = [x[1:] for x in metadata_json['externs']]
-
   # Support older metadata when asmConsts values were lists.  We only use the first element
   # nowadays
   # TODO(sbc): remove this once binaryen has been changed to only emit the single element
@@ -821,19 +821,23 @@ def normalize_line_endings(text):
   return text
 
 
+def clear_struct_info():
+  output_name = shared.Cache.get_lib_name('struct_info.json', varies=False)
+  shared.Cache.erase_file(output_name)
+
+
 def generate_struct_info():
   # If we are running in BOOTSTRAPPING_STRUCT_INFO we don't populate STRUCT_INFO
   # otherwise that would lead to infinite recursion.
   if settings.BOOTSTRAPPING_STRUCT_INFO:
     return
 
-  generated_struct_info_name = 'generated_struct_info.json'
-
   @ToolchainProfiler.profile_block('gen_struct_info')
   def generate_struct_info(out):
     gen_struct_info.main(['-q', '-o', out])
 
-  settings.STRUCT_INFO = shared.Cache.get(generated_struct_info_name, generate_struct_info)
+  output_name = shared.Cache.get_lib_name('struct_info.json', varies=False)
+  settings.STRUCT_INFO = shared.Cache.get(output_name, generate_struct_info)
 
 
 def run(in_wasm, out_wasm, outfile_js, memfile):
