@@ -94,6 +94,11 @@ WebAssembly.Table = function(tableDescriptor) {};
  * @param {GlobalDescriptotr} globalDescriptor
  */
 WebAssembly.Global = function(globalDescriptor) {};
+/**
+ * @constructor
+ * @param {TagDescriptotr} tagDescriptor
+ */
+WebAssembly.Tag = function(tagDescriptor) {};
 /**
  * @constructor
  * @extends {Error}
@@ -296,8 +301,42 @@ var sampleRate;
 /*
  * WebGPU globals
  */
-var GPUValidationError;
+var GPUBufferUsage;
+var GPUColorWrite;
+var GPUMapMode;
+var GPUShaderStage;
+var GPUTextureUsage;
+var GPU;
+var GPUAdapter;
+var GPUBindGroup;
+var GPUBindGroupLayout;
+var GPUBuffer;
+var GPUCanvasContext;
+var GPUCommandBuffer;
+var GPUCommandEncoder;
+var GPUCompilationInfo;
+var GPUCompilationMessage;
+var GPUComputePassEncoder;
+var GPUComputePipeline;
+var GPUDevice;
+var GPUDeviceLostInfo;
+var GPUExternalTexture;
 var GPUOutOfMemoryError;
+var GPUPipelineLayout;
+var GPUQuerySet;
+var GPUQueue;
+var GPURenderBundle;
+var GPURenderBundleEncoder;
+var GPURenderPassEncoder;
+var GPURenderPipeline;
+var GPUSampler;
+var GPUShaderModule;
+var GPUSupportedFeatures;
+var GPUSupportedLimits;
+var GPUTexture;
+var GPUTextureView;
+var GPUUncapturedErrorEvent;
+var GPUValidationError;
 
 /*
  * Avoid closure minifying anything to "id". See #13965
