@@ -23,7 +23,7 @@ var LibraryWebGL2 = {
       }
       return stringiCache[index];
     }
-    switch(name) {
+    switch (name) {
       case 0x1F03 /* GL_EXTENSIONS */:
         var exts = GLctx.getSupportedExtensions() || []; // .getSupportedExtensions() can return null if context is lost, so coerce to empty array.
 #if GL_EXTENSIONS_IN_PREFIXED_FORMAT
@@ -57,6 +57,7 @@ var LibraryWebGL2 = {
 
   glGetInternalformativ__sig: 'viiiii',
   glGetInternalformativ: function(target, internalformat, pname, bufSize, params) {
+#if GL_TRACK_ERRORS
     if (bufSize < 0) {
 #if GL_ASSERTIONS
       err('GL_INVALID_VALUE in glGetInternalformativ(target=' + target + ', internalformat=' + internalformat + ', pname=' + pname + ', bufSize=' + bufSize + ', params=' + params + '): Function called with bufSize < 0!');
@@ -73,10 +74,11 @@ var LibraryWebGL2 = {
       GL.recordError(0x501 /* GL_INVALID_VALUE */);
       return;
     }
+#endif
     var ret = GLctx['getInternalformatParameter'](target, internalformat, pname);
     if (ret === null) return;
     for (var i = 0; i < ret.length && i < bufSize; ++i) {
-      {{{ makeSetValue('params', 'i', 'ret[i]', 'i32') }}};
+      {{{ makeSetValue('params', 'i*4', 'ret[i]', 'i32') }}};
     }
   },
 
@@ -101,18 +103,35 @@ var LibraryWebGL2 = {
   glGetBufferParameteri64v__sig: 'viii',
   glGetBufferParameteri64v__deps: ['$writeI53ToI64'],
   glGetBufferParameteri64v: function(target, value, data) {
+#if GL_TRACK_ERRORS
     if (!data) {
       // GLES2 specification does not specify how to behave if data is a null pointer. Since calling this function does not make sense
       // if data == null, issue a GL error to notify user about it.
 #if GL_ASSERTIONS
-      err('GL_INVALID_VALUE in glGetBufferParameteri64v(target=' + target + ', value=' + value + ', data=0): Function called with null out pointer!');
+      err('GL_INVALID_VALUE in glGetBufferParameteri64v(target=' + target + ', value=' + value + ', data=0): Function called with null out data pointer!');
 #endif
       GL.recordError(0x501 /* GL_INVALID_VALUE */);
       return;
     }
+#endif
     writeI53ToI64(data, GLctx.getBufferParameter(target, value));
   },
 
+  glGetBufferSubData: function(target, offset, size, data) {
+#if GL_TRACK_ERRORS
+    if (!data) {
+      // GLES2 specification does not specify how to behave if data is a null pointer. Since calling this function does not make sense
+      // if data == null, issue a GL error to notify user about it.
+#if GL_ASSERTIONS
+      err('GL_INVALID_VALUE in glGetBufferSubData(target=' + target + ', offset=' + offset + ', size=' + size + ', data=0): Function called with null out data pointer!');
+#endif
+      GL.recordError(0x501 /* GL_INVALID_VALUE */);
+      return;
+    }
+#endif
+    GLctx['getBufferSubData'](target, offset, HEAPU8, data, size);
+  },
+
   glInvalidateFramebuffer__deps: ['$tempFixedLengthArray'],
   glInvalidateFramebuffer__sig: 'viii',
   glInvalidateFramebuffer: function(target, numAttachments, attachments) {
@@ -206,6 +225,7 @@ var LibraryWebGL2 = {
 
   glGetQueryiv__sig: 'viii',
   glGetQueryiv: function(target, pname, params) {
+#if GL_TRACK_ERRORS
     if (!params) {
       // GLES2 specification does not specify how to behave if params is a null pointer. Since calling this function does not make sense
       // if p == null, issue a GL error to notify user about it.
@@ -215,11 +235,13 @@ var LibraryWebGL2 = {
       GL.recordError(0x501 /* GL_INVALID_VALUE */);
       return;
     }
+#endif
     {{{ makeSetValue('params', '0', 'GLctx[\'getQuery\'](target, pname)', 'i32') }}};
   },
 
   glGetQueryObjectuiv__sig: 'viii',
   glGetQueryObjectuiv: function(id, pname, params) {
+#if GL_TRACK_ERRORS
     if (!params) {
       // GLES2 specification does not specify how to behave if params is a null pointer. Since calling this function does not make sense
       // if p == null, issue a GL error to notify user about it.
@@ -229,6 +251,7 @@ var LibraryWebGL2 = {
       GL.recordError(0x501 /* GL_INVALID_VALUE */);
       return;
     }
+#endif
 #if GL_ASSERTIONS
     GL.validateGLObjectID(GL.queries, id, 'glGetQueryObjectuiv', 'id');
 #endif
@@ -317,6 +340,7 @@ var LibraryWebGL2 = {
 
   glGetSamplerParameterfv__sig: 'viii',
   glGetSamplerParameterfv: function(sampler, pname, params) {
+#if GL_TRACK_ERRORS
     if (!params) {
       // GLES3 specification does not specify how to behave if params is a null pointer. Since calling this function does not make sense
       // if p == null, issue a GL error to notify user about it.
@@ -326,12 +350,13 @@ var LibraryWebGL2 = {
       GL.recordError(0x501 /* GL_INVALID_VALUE */);
       return;
     }
-    sampler = GL.samplers[sampler];
-    {{{ makeSetValue('params', '0', 'GLctx[\'getSamplerParameter\'](sampler, pname)', 'float') }}};
+#endif
+    {{{ makeSetValue('params', '0', 'GLctx[\'getSamplerParameter\'](GL.samplers[sampler], pname)', 'float') }}};
   },
 
   glGetSamplerParameteriv__sig: 'viii',
   glGetSamplerParameteriv: function(sampler, pname, params) {
+#if GL_TRACK_ERRORS
     if (!params) {
       // GLES3 specification does not specify how to behave if params is a null pointer. Since calling this function does not make sense
       // if p == null, issue a GL error to notify user about it.
@@ -341,8 +366,8 @@ var LibraryWebGL2 = {
       GL.recordError(0x501 /* GL_INVALID_VALUE */);
       return;
     }
-    sampler = GL.samplers[sampler];
-    {{{ makeSetValue('params', '0', 'GLctx[\'getSamplerParameter\'](sampler, pname)', 'i32') }}};
+#endif
+    {{{ makeSetValue('params', '0', 'GLctx[\'getSamplerParameter\'](GL.samplers[sampler], pname)', 'i32') }}};
   },
 
   // Transform Feedback
@@ -416,6 +441,7 @@ var LibraryWebGL2 = {
 
   $emscriptenWebGLGetIndexed__deps: ['$writeI53ToI64'],
   $emscriptenWebGLGetIndexed: function(target, index, data, type) {
+#if GL_TRACK_ERRORS
     if (!data) {
       // GLES2 specification does not specify how to behave if data is a null pointer. Since calling this function does not make sense
       // if data == null, issue a GL error to notify user about it.
@@ -425,6 +451,7 @@ var LibraryWebGL2 = {
       GL.recordError(0x501 /* GL_INVALID_VALUE */);
       return;
     }
+#endif
     var result = GLctx['getIndexedParameter'](target, index);
     var ret;
     switch (typeof result) {
@@ -510,6 +537,7 @@ var LibraryWebGL2 = {
 #if GL_ASSERTIONS
     GL.validateGLObjectID(GL.programs, program, 'glGetUniformIndices', 'program');
 #endif
+#if GL_TRACK_ERRORS
     if (!uniformIndices) {
       // GLES2 specification does not specify how to behave if uniformIndices is a null pointer. Since calling this function does not make sense
       // if uniformIndices == null, issue a GL error to notify user about it.
@@ -523,6 +551,7 @@ var LibraryWebGL2 = {
       GL.recordError(0x501 /* GL_INVALID_VALUE */);
       return;
     }
+#endif
     program = GL.programs[program];
     var names = [];
     for (var i = 0; i < uniformCount; i++)
@@ -542,6 +571,7 @@ var LibraryWebGL2 = {
 #if GL_ASSERTIONS
     GL.validateGLObjectID(GL.programs, program, 'glGetActiveUniformsiv', 'program');
 #endif
+#if GL_TRACK_ERRORS
     if (!params) {
       // GLES2 specification does not specify how to behave if params is a null pointer. Since calling this function does not make sense
       // if params == null, issue a GL error to notify user about it.
@@ -555,6 +585,7 @@ var LibraryWebGL2 = {
       GL.recordError(0x501 /* GL_INVALID_VALUE */);
       return;
     }
+#endif
     program = GL.programs[program];
     var ids = [];
     for (var i = 0; i < uniformCount; i++) {
@@ -580,6 +611,7 @@ var LibraryWebGL2 = {
 
   glGetActiveUniformBlockiv__sig: 'viiii',
   glGetActiveUniformBlockiv: function(program, uniformBlockIndex, pname, params) {
+#if GL_TRACK_ERRORS
     if (!params) {
       // GLES2 specification does not specify how to behave if params is a null pointer. Since calling this function does not make sense
       // if params == null, issue a GL error to notify user about it.
@@ -589,26 +621,26 @@ var LibraryWebGL2 = {
       GL.recordError(0x501 /* GL_INVALID_VALUE */);
       return;
     }
+#endif
 #if GL_ASSERTIONS
     GL.validateGLObjectID(GL.programs, program, 'glGetActiveUniformBlockiv', 'program');
 #endif
     program = GL.programs[program];
 
-    switch(pname) {
-      case 0x8A41: /* GL_UNIFORM_BLOCK_NAME_LENGTH */
-        var name = GLctx['getActiveUniformBlockName'](program, uniformBlockIndex);
-        {{{ makeSetValue('params', 0, 'name.length+1', 'i32') }}};
-        return;
-      default:
-        var result = GLctx['getActiveUniformBlockParameter'](program, uniformBlockIndex, pname);
-        if (!result) return; // If an error occurs, nothing will be written to params.
-        if (typeof result == 'number') {
-          {{{ makeSetValue('params', '0', 'result', 'i32') }}};
-        } else {
-          for (var i = 0; i < result.length; i++) {
-            {{{ makeSetValue('params', 'i*4', 'result[i]', 'i32') }}};
-          }
-        }
+    if (pname == 0x8A41 /* GL_UNIFORM_BLOCK_NAME_LENGTH */) {
+      var name = GLctx['getActiveUniformBlockName'](program, uniformBlockIndex);
+      {{{ makeSetValue('params', 0, 'name.length+1', 'i32') }}};
+      return;
+    }
+
+    var result = GLctx['getActiveUniformBlockParameter'](program, uniformBlockIndex, pname);
+    if (result === null) return; // If an error occurs, nothing should be written to params.
+    if (pname == 0x8A43 /*GL_UNIFORM_BLOCK_ACTIVE_UNIFORM_INDICES*/) {
+      for (var i = 0; i < result.length; i++) {
+        {{{ makeSetValue('params', 'i*4', 'result[i]', 'i32') }}};
+      }
+    } else {
+      {{{ makeSetValue('params', '0', 'result', 'i32') }}};
     }
   },
 
@@ -711,6 +743,7 @@ var LibraryWebGL2 = {
 
   glGetSynciv__sig: 'viiiii',
   glGetSynciv: function(sync, pname, bufSize, length, values) {
+#if GL_TRACK_ERRORS
     if (bufSize < 0) {
       // GLES3 specification does not specify how to behave if bufSize < 0, however in the spec wording for glGetInternalformativ, it does say that GL_INVALID_VALUE should be raised,
       // so raise GL_INVALID_VALUE here as well.
@@ -729,9 +762,12 @@ var LibraryWebGL2 = {
       GL.recordError(0x501 /* GL_INVALID_VALUE */);
       return;
     }
+#endif
     var ret = GLctx.getSyncParameter(GL.syncs[sync], pname);
-    {{{ makeSetValue('length', '0', 'ret', 'i32') }}};
-    if (ret !== null && length) {{{ makeSetValue('length', '0', '1', 'i32') }}}; // Report a single value outputted.
+    if (ret !== null) {
+      {{{ makeSetValue('values', '0', 'ret', 'i32') }}};
+      if (length) {{{ makeSetValue('length', '0', '1', 'i32') }}}; // Report a single value outputted.
+    }
   },
 
   glIsSync__sig: 'ii',
@@ -768,125 +804,139 @@ var LibraryWebGL2 = {
   glGetVertexAttribIuiv: 'glGetVertexAttribIiv',
 
   glUniform1ui__sig: 'vii',
+  glUniform1ui__deps: ['$webglGetUniformLocation'],
   glUniform1ui: function(location, v0) {
 #if GL_ASSERTIONS
-    GL.validateGLObjectID(GL.uniforms, location, 'glUniform1ui', 'location');
+    GL.validateGLObjectID(GLctx.currentProgram.uniformLocsById, location, 'glUniform1ui', 'location');
 #endif
-    GLctx.uniform1ui(GL.uniforms[location], v0);
+    GLctx.uniform1ui(webglGetUniformLocation(location), v0);
   },
 
   glUniform2ui__sig: 'viii',
+  glUniform2ui__deps: ['$webglGetUniformLocation'],
   glUniform2ui: function(location, v0, v1) {
 #if GL_ASSERTIONS
-    GL.validateGLObjectID(GL.uniforms, location, 'glUniform2ui', 'location');
+    GL.validateGLObjectID(GLctx.currentProgram.uniformLocsById, location, 'glUniform2ui', 'location');
 #endif
-    GLctx.uniform2ui(GL.uniforms[location], v0, v1);
+    GLctx.uniform2ui(webglGetUniformLocation(location), v0, v1);
   },
 
   glUniform3ui__sig: 'viiii',
+  glUniform3ui__deps: ['$webglGetUniformLocation'],
   glUniform3ui: function(location, v0, v1, v2) {
 #if GL_ASSERTIONS
-    GL.validateGLObjectID(GL.uniforms, location, 'glUniform3ui', 'location');
+    GL.validateGLObjectID(GLctx.currentProgram.uniformLocsById, location, 'glUniform3ui', 'location');
 #endif
-    GLctx.uniform3ui(GL.uniforms[location], v0, v1, v2);
+    GLctx.uniform3ui(webglGetUniformLocation(location), v0, v1, v2);
   },
 
   glUniform4ui__sig: 'viiiii',
+  glUniform4ui__deps: ['$webglGetUniformLocation'],
   glUniform4ui: function(location, v0, v1, v2, v3) {
 #if GL_ASSERTIONS
-    GL.validateGLObjectID(GL.uniforms, location, 'glUniform4ui', 'location');
+    GL.validateGLObjectID(GLctx.currentProgram.uniformLocsById, location, 'glUniform4ui', 'location');
 #endif
-    GLctx.uniform4ui(GL.uniforms[location], v0, v1, v2, v3);
+    GLctx.uniform4ui(webglGetUniformLocation(location), v0, v1, v2, v3);
   },
 
   glUniform1uiv__sig: 'viii',
+  glUniform1uiv__deps: ['$webglGetUniformLocation'],
   glUniform1uiv: function(location, count, value) {
 #if GL_ASSERTIONS
-    GL.validateGLObjectID(GL.uniforms, location, 'glUniform1uiv', 'location');
+    GL.validateGLObjectID(GLctx.currentProgram.uniformLocsById, location, 'glUniform1uiv', 'location');
     assert((value & 3) == 0, 'Pointer to integer data passed to glUniform1uiv must be aligned to four bytes!');
 #endif
-    GLctx.uniform1uiv(GL.uniforms[location], HEAPU32, value>>2, count);
+    GLctx.uniform1uiv(webglGetUniformLocation(location), HEAPU32, value>>2, count);
   },
 
   glUniform2uiv__sig: 'viii',
+  glUniform2uiv__deps: ['$webglGetUniformLocation'],
   glUniform2uiv: function(location, count, value) {
 #if GL_ASSERTIONS
-    GL.validateGLObjectID(GL.uniforms, location, 'glUniform2uiv', 'location');
+    GL.validateGLObjectID(GLctx.currentProgram.uniformLocsById, location, 'glUniform2uiv', 'location');
     assert((value & 3) == 0, 'Pointer to integer data passed to glUniform2uiv must be aligned to four bytes!');
 #endif
-    GLctx.uniform2uiv(GL.uniforms[location], HEAPU32, value>>2, count*2);
+    GLctx.uniform2uiv(webglGetUniformLocation(location), HEAPU32, value>>2, count*2);
   },
 
   glUniform3uiv__sig: 'viii',
+  glUniform3uiv__deps: ['$webglGetUniformLocation'],
   glUniform3uiv: function(location, count, value) {
 #if GL_ASSERTIONS
-    GL.validateGLObjectID(GL.uniforms, location, 'glUniform3uiv', 'location');
+    GL.validateGLObjectID(GLctx.currentProgram.uniformLocsById, location, 'glUniform3uiv', 'location');
     assert((value & 3) == 0, 'Pointer to integer data passed to glUniform3uiv must be aligned to four bytes!');
 #endif
-    GLctx.uniform3uiv(GL.uniforms[location], HEAPU32, value>>2, count*3);
+    GLctx.uniform3uiv(webglGetUniformLocation(location), HEAPU32, value>>2, count*3);
   },
 
   glUniform4uiv__sig: 'viii',
+  glUniform4uiv__deps: ['$webglGetUniformLocation'],
   glUniform4uiv: function(location, count, value) {
 #if GL_ASSERTIONS
-    GL.validateGLObjectID(GL.uniforms, location, 'glUniform4uiv', 'location');
+    GL.validateGLObjectID(GLctx.currentProgram.uniformLocsById, location, 'glUniform4uiv', 'location');
     assert((value & 3) == 0, 'Pointer to integer data passed to glUniform4uiv must be aligned to four bytes!');
 #endif
-    GLctx.uniform4uiv(GL.uniforms[location], HEAPU32, value>>2, count*4);
+    GLctx.uniform4uiv(webglGetUniformLocation(location), HEAPU32, value>>2, count*4);
   },
 
   glUniformMatrix2x3fv__sig: 'viiii',
+  glUniformMatrix2x3fv__deps: ['$webglGetUniformLocation'],
   glUniformMatrix2x3fv: function(location, count, transpose, value) {
 #if GL_ASSERTIONS
-    GL.validateGLObjectID(GL.uniforms, location, 'glUniformMatrix2x3fv', 'location');
+    GL.validateGLObjectID(GLctx.currentProgram.uniformLocsById, location, 'glUniformMatrix2x3fv', 'location');
     assert((value & 3) == 0, 'Pointer to float data passed to glUniformMatrix2x3fv must be aligned to four bytes!');
 #endif
-    GLctx.uniformMatrix2x3fv(GL.uniforms[location], !!transpose, HEAPF32, value>>2, count*6);
+    GLctx.uniformMatrix2x3fv(webglGetUniformLocation(location), !!transpose, HEAPF32, value>>2, count*6);
   },
 
   glUniformMatrix3x2fv__sig: 'viiii',
+  glUniformMatrix3x2fv__deps: ['$webglGetUniformLocation'],
   glUniformMatrix3x2fv: function(location, count, transpose, value) {
 #if GL_ASSERTIONS
-    GL.validateGLObjectID(GL.uniforms, location, 'glUniformMatrix3x2fv', 'location');
+    GL.validateGLObjectID(GLctx.currentProgram.uniformLocsById, location, 'glUniformMatrix3x2fv', 'location');
     assert((value & 3) == 0, 'Pointer to float data passed to glUniformMatrix3x2fv must be aligned to four bytes!');
 #endif
-    GLctx.uniformMatrix3x2fv(GL.uniforms[location], !!transpose, HEAPF32, value>>2, count*6);
+    GLctx.uniformMatrix3x2fv(webglGetUniformLocation(location), !!transpose, HEAPF32, value>>2, count*6);
   },
 
   glUniformMatrix2x4fv__sig: 'viiii',
+  glUniformMatrix2x4fv__deps: ['$webglGetUniformLocation'],
   glUniformMatrix2x4fv: function(location, count, transpose, value) {
 #if GL_ASSERTIONS
-    GL.validateGLObjectID(GL.uniforms, location, 'glUniformMatrix2x4fv', 'location');
+    GL.validateGLObjectID(GLctx.currentProgram.uniformLocsById, location, 'glUniformMatrix2x4fv', 'location');
     assert((value & 3) == 0, 'Pointer to float data passed to glUniformMatrix2x4fv must be aligned to four bytes!');
 #endif
-    GLctx.uniformMatrix2x4fv(GL.uniforms[location], !!transpose, HEAPF32, value>>2, count*8);
+    GLctx.uniformMatrix2x4fv(webglGetUniformLocation(location), !!transpose, HEAPF32, value>>2, count*8);
   },
 
   glUniformMatrix4x2fv__sig: 'viiii',
+  glUniformMatrix4x2fv__deps: ['$webglGetUniformLocation'],
   glUniformMatrix4x2fv: function(location, count, transpose, value) {
 #if GL_ASSERTIONS
-    GL.validateGLObjectID(GL.uniforms, location, 'glUniformMatrix4x2fv', 'location');
+    GL.validateGLObjectID(GLctx.currentProgram.uniformLocsById, location, 'glUniformMatrix4x2fv', 'location');
     assert((value & 3) == 0, 'Pointer to float data passed to glUniformMatrix4x2fv must be aligned to four bytes!');
 #endif
-    GLctx.uniformMatrix4x2fv(GL.uniforms[location], !!transpose, HEAPF32, value>>2, count*8);
+    GLctx.uniformMatrix4x2fv(webglGetUniformLocation(location), !!transpose, HEAPF32, value>>2, count*8);
   },
 
   glUniformMatrix3x4fv__sig: 'viiii',
+  glUniformMatrix3x4fv__deps: ['$webglGetUniformLocation'],
   glUniformMatrix3x4fv: function(location, count, transpose, value) {
 #if GL_ASSERTIONS
-    GL.validateGLObjectID(GL.uniforms, location, 'glUniformMatrix3x4fv', 'location');
+    GL.validateGLObjectID(GLctx.currentProgram.uniformLocsById, location, 'glUniformMatrix3x4fv', 'location');
     assert((value & 3) == 0, 'Pointer to float data passed to glUniformMatrix3x4fv must be aligned to four bytes!');
 #endif
-    GLctx.uniformMatrix3x4fv(GL.uniforms[location], !!transpose, HEAPF32, value>>2, count*12);
+    GLctx.uniformMatrix3x4fv(webglGetUniformLocation(location), !!transpose, HEAPF32, value>>2, count*12);
   },
 
   glUniformMatrix4x3fv__sig: 'viiii',
+  glUniformMatrix4x3fv__deps: ['$webglGetUniformLocation'],
   glUniformMatrix4x3fv: function(location, count, transpose, value) {
 #if GL_ASSERTIONS
-    GL.validateGLObjectID(GL.uniforms, location, 'glUniformMatrix4x3fv', 'location');
+    GL.validateGLObjectID(GLctx.currentProgram.uniformLocsById, location, 'glUniformMatrix4x3fv', 'location');
     assert((value & 3) == 0, 'Pointer to float data passed to glUniformMatrix4x3fv must be aligned to four bytes!');
 #endif
-    GLctx.uniformMatrix4x3fv(GL.uniforms[location], !!transpose, HEAPF32, value>>2, count*12);
+    GLctx.uniformMatrix4x3fv(webglGetUniformLocation(location), !!transpose, HEAPF32, value>>2, count*12);
   },
 
   glVertexAttribI4iv__sig: 'vii',
