@@ -39,12 +39,8 @@ function genArgSequence(n) {
   return args;
 }
 
-var Functions = {
-  // All functions that will be implemented in this file. Maps id to signature
-  implementedFunctions: {},
-  // functions added from the library. value 2 means asmLibraryFunction
-  libraryFunctions: {},
-};
+// List of functions that were added from the library.
+var libraryFunctions = [];
 
 var LibraryManager = {
   library: null,
@@ -53,7 +49,7 @@ var LibraryManager = {
   libraries: [],
 
   has: function(name) {
-    return this.libraries.indexOf(name) >= 0;
+    return this.libraries.includes(name);
   },
 
   load: function() {
@@ -65,20 +61,20 @@ var LibraryManager = {
       'library_formatString.js',
       'library_math.js',
       'library_path.js',
-      'library_signals.js',
       'library_syscall.js',
       'library_html5.js',
       'library_stack_trace.js',
       'library_wasi.js',
       'library_int53.js',
-      'library_dylink.js'
+      'library_dylink.js',
+      'library_eventloop.js',
     ];
 
-    if (!EXCEPTION_HANDLING) {
-      if (!DISABLE_EXCEPTION_THROWING) {
-        libraries.push('library_exceptions.js');
-      } else {
+    if (LINK_AS_CXX && !EXCEPTION_HANDLING) {
+      if (DISABLE_EXCEPTION_THROWING) {
         libraries.push('library_exceptions_stub.js');
+      } else {
+        libraries.push('library_exceptions.js');
       }
     }
 
@@ -90,6 +86,7 @@ var LibraryManager = {
       libraries.push('library_strings.js');
     } else {
       libraries.push('library_browser.js');
+      libraries.push('library_wget.js');
     }
 
     if (USE_PTHREADS) { // TODO: Currently WebGL proxying makes pthreads library depend on WebGL.
@@ -109,12 +106,14 @@ var LibraryManager = {
 
       if (NODERAWFS) {
         // NODERAWFS requires NODEFS
-        if (SYSTEM_JS_LIBRARIES.indexOf('library_nodefs.js') < 0) {
+        if (!JS_LIBRARIES.includes('library_nodefs.js')) {
           libraries.push('library_nodefs.js');
         }
         libraries.push('library_noderawfs.js');
       }
-    }
+    } else if (WASMFS) {
+      libraries.push('library_wasmfs.js');
+    } 
 
     // Additional JS libraries (without AUTO_JS_LIBRARIES, link to these explicitly via -lxxx.js)
     if (AUTO_JS_LIBRARIES) {
@@ -145,7 +144,7 @@ var LibraryManager = {
     }
 
     // Add any explicitly specified system JS libraries to link to, add those to link.
-    libraries = libraries.concat(SYSTEM_JS_LIBRARIES)
+    libraries = libraries.concat(JS_LIBRARIES)
 
     if (LZ4) {
       libraries.push('library_lz4.js');
@@ -155,7 +154,7 @@ var LibraryManager = {
       libraries.push('library_webgl2.js');
     }
 
-    if (GL_EXPLICIT_UNIFORM_LOCATION) {
+    if (GL_EXPLICIT_UNIFORM_LOCATION || GL_EXPLICIT_UNIFORM_BINDING) {
       libraries.push('library_c_preprocessor.js');
     }
 
@@ -172,7 +171,6 @@ var LibraryManager = {
       libraries = [
         'library_bootstrap.js',
         'library_formatString.js',
-        'library_stack_trace.js',
         'library_int53.js',
       ];
     }
@@ -182,53 +180,77 @@ var LibraryManager = {
     }
 
     // Deduplicate libraries to avoid processing any library file multiple times
-    libraries = libraries.filter(function(item, pos) {
-      return libraries.indexOf(item) == pos;
-    });
+    libraries = libraries.filter((item, pos) => libraries.indexOf(item) == pos);
 
     // Save the list for has() queries later.
     this.libraries = libraries;
 
     for (var filename of libraries) {
+      var isUserLibrary = nodePath.isAbsolute(filename);
+      if (VERBOSE) {
+        if (isUserLibrary) {
+          printErr('processing user library: ' + filename);
+        } else {
+          printErr('processing system library: ' + filename);
+        }
+      }
       var src = read(filename);
+      var origLibrary = undefined;
       var processed = undefined;
+      // When we parse user libraries also set `__user` attribute
+      // on each element so that we can distinguish them later.
+      if (isUserLibrary) {
+        origLibrary = this.library
+        this.library = new Proxy(this.library, {
+          set: (target, prop, value) => {
+            target[prop] = value;
+            if (!isJsLibraryConfigIdentifier(prop)) {
+              target[prop + '__user'] = true;
+            }
+          }
+        });
+      }
       try {
         processed = processMacros(preprocess(src, filename));
         eval(processed);
       } catch(e) {
-        var details = [e, e.lineNumber ? 'line number: ' + e.lineNumber : ''];
+        var details = [e, e.lineNumber ? `line number: ${e.lineNumber}` : ''];
         if (VERBOSE) {
           details.push((e.stack || "").toString().replace('Object.<anonymous>', filename));
         }
         if (processed) {
-          error('failure to execute js library "' + filename + '": ' + details);
+          error(`failure to execute js library "${filename}": ${details}`);
           if (VERBOSE) {
-            error('preprocessed source (you can run a js engine on this to get a clearer error message sometimes):\n=============\n' + processed + '\n=============');
+            error(`preprocessed source (you can run a js engine on this to get a clearer error message sometimes):\n=============\n${processed}\n=============`);
           } else {
             error('use -s VERBOSE to see more details')
           }
         } else {
-          error('failure to process js library "' + filename + '": ' + details);
+          error(`failure to process js library "${filename}": ${details}`);
           if (VERBOSE) {
-            error('original source:\n=============\n' + src + '\n=============');
+            error(`original source:\n=============\n${src}\n=============`);
           } else {
             error('use -s VERBOSE to see more details')
           }
         }
         throw e;
+      } finally {
+        if (origLibrary) {
+          this.library = origLibrary;
+        }
       }
     }
 
     // apply synonyms. these are typically not speed-sensitive, and doing it
     // this way makes it possible to not include hacks in the compiler
     // (and makes it simpler to switch between SDL versions, fastcomp and non-fastcomp, etc.).
-    var lib = LibraryManager.library;
+    var lib = this.library;
     libloop: for (var x in lib) {
       if (isJsLibraryConfigIdentifier(x)) {
         var index = x.lastIndexOf('__');
         var basename = x.slice(0, index);
         if (!(basename in lib)) {
-          error('Missing library element `' + basename + '` for library config `' + x + '`');
+          error(`Missing library element '${basename}' for library config '${x}'`);
         }
         continue;
       }
@@ -246,7 +268,7 @@ var LibraryManager = {
           // implemented.
           function testStringType(sig) {
             if (typeof lib[sig] !== 'undefined' && typeof typeof lib[sig] !== 'string') {
-              error(sig + ' should be a string! (was ' + typeof lib[sig]);
+              error(`${sig} should be a string! (was ${typeof lib[sig]})`);
             }
           }
           var aliasSig = x + '__sig';
@@ -254,12 +276,12 @@ var LibraryManager = {
           testStringType(aliasSig);
           testStringType(targetSig);
           if (typeof lib[aliasSig] === 'string' && typeof lib[targetSig] === 'string' && lib[aliasSig] != lib[targetSig]) {
-            error(aliasSig + ' (' + lib[aliasSig] + ')  differs from ' + targetSig + ' (' + lib[targetSig] + ')');
+            error(`${aliasSig} (${lib[aliasSig]}) differs from ${targetSig} (${lib[targetSig]})`);
           }
 
           var sig = lib[aliasSig] || lib[targetSig];
           if (typeof sig !== 'string') {
-            error('Function ' + x + ' aliases to target function ' + target + ', but neither the alias or the target provide a signature. Please add a ' + targetSig + ": 'vifj...' annotation or a " + aliasSig + ": 'vifj...' annotation to describe the type of function forwarding that is needed!");
+            error(`Function ${x} aliases to target function ${target}, but neither the alias or the target provide a signature. Please add a ${targetSig}: 'vifj...' annotation or a ${aliasSig}: 'vifj...' annotation to describe the type of function forwarding that is needed!`);
           }
 
           // If only one of the target or the alias specifies a sig then copy
@@ -280,7 +302,7 @@ var LibraryManager = {
           }
           var ret = sig == 'v' ? '' : 'return ';
           var args = genArgSequence(argCount).join(',');
-          lib[x] = new Function(args, ret + '_' + target + '(' + args + ');');
+          lib[x] = new Function(args, `${ret}_${target}(${args});`);
 
           if (!lib[x + '__deps']) lib[x + '__deps'] = [];
           lib[x + '__deps'].push(target);
@@ -290,26 +312,6 @@ var LibraryManager = {
 
     this.loaded = true;
   },
-
-  // Given an ident, see if it is an alias for something, and so forth, returning
-  // the earliest ancestor (the root)
-  getRootIdent: function(ident) {
-    if (!this.library) return null;
-    var ret = LibraryManager.library[ident];
-    if (!ret) return null;
-    var last = ident;
-    while (typeof ret === 'string') {
-      last = ret;
-      ret = LibraryManager.library[ret];
-    }
-    return last;
-  },
-
-  isStubFunction: function(ident) {
-    var libCall = LibraryManager.library[ident.substr(1)];
-    return typeof libCall === 'function' && libCall.toString().replace(/\s/g, '') === 'function(){}'
-                                         && !(ident in Functions.implementedFunctions);
-  }
 };
 
 if (!BOOTSTRAPPING_STRUCT_INFO) {
@@ -324,13 +326,11 @@ if (!BOOTSTRAPPING_STRUCT_INFO) {
 
 // Safe way to access a C define. We check that we don't add library functions with missing defines.
 function cDefine(key) {
-	if (key in C_DEFINES) return C_DEFINES[key];
-	throw 'Missing C define ' + key + '! If you just added it to struct_info.json, you need to ./emcc --clear-cache';
+  if (key in C_DEFINES) return C_DEFINES[key];
+  throw `Missing C define ${key}! If you just added it to struct_info.json, you need to ./emcc --clear-cache`;
 }
 
-var EXPORTED_RUNTIME_METHODS_SET = set(EXPORTED_RUNTIME_METHODS.concat(EXTRA_EXPORTED_RUNTIME_METHODS));
-EXPORTED_RUNTIME_METHODS = unset(EXPORTED_RUNTIME_METHODS_SET);
-EXTRA_EXPORTED_RUNTIME_METHODS = [];
+var EXPORTED_RUNTIME_METHODS_SET = set(EXPORTED_RUNTIME_METHODS);
 
 function isFSPrefixed(name) {
   return name.length > 3 && name[0] === 'F' && name[1] === 'S' && name[2] === '_';
@@ -367,7 +367,7 @@ function exportRuntime() {
       } else if (exported === 'printErr') {
         exported = 'err';
       }
-      return 'Module["' + name + '"] = ' + exported + ';';
+      return `Module["${name}"] = ${exported};`;
     }
     // do not export it. but if ASSERTIONS, emit a
     // stub with an error, so the user gets a message
@@ -381,9 +381,9 @@ function exportRuntime() {
         extra = '. Alternatively, forcing filesystem support (-s FORCE_FILESYSTEM=1) can export this for you';
       }
       if (!isNumber) {
-        return 'if (!Object.getOwnPropertyDescriptor(Module, "' + name + '")) Module["' + name + '"] = function() { abort("\'' + name + '\' was not exported. add it to EXTRA_EXPORTED_RUNTIME_METHODS (see the FAQ)' + extra + '") };';
+        return `if (!Object.getOwnPropertyDescriptor(Module, "${name}")) Module["${name}"] = function() { abort("'${name}' was not exported. add it to EXPORTED_RUNTIME_METHODS (see the FAQ)${extra}") };`;
       } else {
-        return 'if (!Object.getOwnPropertyDescriptor(Module, "' + name + '")) Object.defineProperty(Module, "' + name + '", { configurable: true, get: function() { abort("\'' + name + '\' was not exported. add it to EXTRA_EXPORTED_RUNTIME_METHODS (see the FAQ)' + extra + '") } });';
+        return `if (!Object.getOwnPropertyDescriptor(Module, "${name}")) Object.defineProperty(Module, "${name}", { configurable: true, get: function() { abort("'${name}' was not exported. add it to EXPORTED_RUNTIME_METHODS (see the FAQ)${extra}") } });`;
       }
     }
     return '';
@@ -393,7 +393,7 @@ function exportRuntime() {
     return maybeExport(name, true);
   }
 
-  // All possible runtime elements to export
+  // All possible runtime elements that can be exported
   var runtimeElements = [
     'intArrayFromString',
     'intArrayToString',
@@ -434,7 +434,6 @@ function exportRuntime() {
     'removeFunction',
     'getFuncWrapper',
     'prettyPrint',
-    'makeBigInt',
     'dynCall',
     'getCompilerSetting',
     'print',
@@ -443,6 +442,7 @@ function exportRuntime() {
     'setTempRet0',
     'callMain',
     'abort',
+    'keepRuntimeAlive',
   ];
 
   if (USE_OFFSET_CONVERTER) {
@@ -496,7 +496,7 @@ function exportRuntime() {
       threadExports.push('ExitStatus');
     }
 
-    threadExports.forEach(function(x) {
+    threadExports.forEach(x => {
       EXPORTED_RUNTIME_METHODS_SET[x] = 1;
       runtimeElements.push(x);
     });
@@ -522,46 +522,13 @@ function exportRuntime() {
   if (ASSERTIONS) {
     // check all exported things exist, warn about typos
     for (var name in EXPORTED_RUNTIME_METHODS_SET) {
-      if (runtimeElements.indexOf(name) < 0 &&
-          runtimeNumbers.indexOf(name) < 0) {
-        printErr('warning: invalid item (maybe a typo?) in EXPORTED_RUNTIME_METHODS: ' + name);
+      if (!runtimeElements.includes(name) && !runtimeNumbers.includes(name)) {
+        printErr(`warning: invalid item (maybe a typo?) in EXPORTED_RUNTIME_METHODS: ${name}`);
       }
     }
   }
-  var exports = runtimeElements.map(function(name) { return maybeExport(name); });
-  exports = exports.concat(runtimeNumbers.map(function(name) { return maybeExportNumber(name); }));
-  exports = exports.filter(function(name) { return name != '' });
+  var exports = runtimeElements.map(name => maybeExport(name));
+  exports = exports.concat(runtimeNumbers.map(name => maybeExportNumber(name)));
+  exports = exports.filter(name => name != '');
   return exports.join('\n');
 }
-
-var PassManager = {
-  serialize: function() {
-    print('\n//FORWARDED_DATA:' + JSON.stringify({
-      Functions: Functions,
-      EXPORTED_FUNCTIONS: EXPORTED_FUNCTIONS,
-      ATINITS: ATINITS.join('\n'),
-      ATMAINS: ATMAINS.join('\n'),
-      ATEXITS: ATEXITS.join('\n'),
-    }));
-  },
-  load: function(json) {
-    var data = JSON.parse(json);
-    for (var i in data.Types) {
-      Types[i] = data.Types[i];
-    }
-    for (var i in data.Variables) {
-      Variables[i] = data.Variables[i];
-    }
-    for (var i in data.Functions) {
-      Functions[i] = data.Functions[i];
-    }
-    EXPORTED_FUNCTIONS = data.EXPORTED_FUNCTIONS;
-    /*
-    print('\n//LOADED_DATA:' + JSON.stringify({
-      Types: Types,
-      Variables: Variables,
-      Functions: Functions
-    }));
-    */
-  }
-};
