@@ -32,7 +32,7 @@ var ENVIRONMENT_IS_NODE = typeof process === 'object';
 var ENVIRONMENT_IS_SHELL = typeof read === 'function';
 #endif
 
-#if ASSERTIONS
+#if ASSERTIONS || USE_PTHREADS
 #if !ENVIRONMENT_MAY_BE_NODE && !ENVIRONMENT_MAY_BE_SHELL
 var ENVIRONMENT_IS_WEB = true
 #elif ENVIRONMENT && !ENVIRONMENT.includes(',')
@@ -43,10 +43,8 @@ var ENVIRONMENT_IS_WEB = !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_SHELL;
 var ENVIRONMENT_IS_WEB = !ENVIRONMENT_IS_SHELL;
 #else
 var ENVIRONMENT_IS_WEB = !ENVIRONMENT_IS_NODE;
-#else
-
 #endif
-#endif // ASSERTIONS
+#endif // ASSERTIONS || USE_PTHREADS
 
 #if ASSERTIONS && ENVIRONMENT_MAY_BE_NODE && ENVIRONMENT_MAY_BE_SHELL
 if (ENVIRONMENT_IS_NODE && ENVIRONMENT_IS_SHELL) {
