@@ -204,15 +204,19 @@ class EmscriptenBenchmarker(Benchmarker):
       EMCC, filename,
       OPTIMIZATIONS,
       '-s', 'INITIAL_MEMORY=256MB',
-      '-s', 'FILESYSTEM=0',
-      '--closure=1',
-      '-s', 'MINIMAL_RUNTIME',
       '-s', 'ENVIRONMENT=node,shell',
       '-s', 'BENCHMARK=%d' % (1 if IGNORE_COMPILATION and not has_output_parser else 0),
       '-o', final
-    ] + shared_args + emcc_args + LLVM_FEATURE_FLAGS + self.extra_args
-    if 'FORCE_FILESYSTEM' in cmd:
-      cmd = [arg if arg != 'FILESYSTEM=0' else 'FILESYSTEM=1' for arg in cmd]
+    ] + shared_args + LLVM_FEATURE_FLAGS
+    if common.EMTEST_FORCE64:
+      cmd += ['--profiling']
+    else:
+      cmd += ['--closure=1', '-sMINIMAL_RUNTIME']
+    # add additional emcc args at the end, which may override other things
+    # above, such as minimal runtime
+    cmd += emcc_args + self.extra_args
+    if 'FORCE_FILESYSTEM' not in cmd:
+      cmd += ['-s', 'FILESYSTEM=0']
     if PROFILING:
       cmd += ['--profiling-funcs']
     self.cmd = cmd
@@ -353,10 +357,13 @@ class CheerpBenchmarker(Benchmarker):
 
 # Benchmarkers
 
-benchmarkers = [
-  NativeBenchmarker('clang', [CLANG_CC], [CLANG_CXX]),
-  # NativeBenchmarker('gcc',   ['gcc', '-no-pie'],  ['g++', '-no-pie'])
-]
+benchmarkers = []
+
+if not common.EMTEST_FORCE64:
+  benchmarkers += [
+    NativeBenchmarker('clang', [CLANG_CC], [CLANG_CXX]),
+    # NativeBenchmarker('gcc',   ['gcc', '-no-pie'],  ['g++', '-no-pie'])
+  ]
 
 if config.V8_ENGINE and config.V8_ENGINE in config.JS_ENGINES:
   # avoid the baseline compiler running, because it adds a lot of noise
@@ -364,11 +371,16 @@ if config.V8_ENGINE and config.V8_ENGINE in config.JS_ENGINES:
   # mattering as much as the actual benchmark)
   aot_v8 = config.V8_ENGINE + ['--no-liftoff']
   default_v8_name = os.environ.get('EMBENCH_NAME') or 'v8'
-  benchmarkers += [
-    EmscriptenBenchmarker(default_v8_name, aot_v8),
-    EmscriptenBenchmarker(default_v8_name + '-lto', aot_v8, ['-flto']),
-    # EmscriptenWasm2CBenchmarker('wasm2c')
-  ]
+  if common.EMTEST_FORCE64:
+    benchmarkers += [
+      EmscriptenBenchmarker(default_v8_name, aot_v8, ['-sMEMORY64=2']),
+    ]
+  else:
+    benchmarkers += [
+      EmscriptenBenchmarker(default_v8_name, aot_v8),
+      EmscriptenBenchmarker(default_v8_name + '-lto', aot_v8, ['-flto']),
+      # EmscriptenWasm2CBenchmarker('wasm2c')
+    ]
   if os.path.exists(CHEERP_BIN):
     benchmarkers += [
       # CheerpBenchmarker('cheerp-v8-wasm', aot_v8),
@@ -385,9 +397,14 @@ if config.SPIDERMONKEY_ENGINE and config.SPIDERMONKEY_ENGINE in config.JS_ENGINE
     ]
 
 if config.NODE_JS and config.NODE_JS in config.JS_ENGINES:
-  benchmarkers += [
-    # EmscriptenBenchmarker('Node.js', config.NODE_JS),
-  ]
+  if common.EMTEST_FORCE64:
+    benchmarkers += [
+      EmscriptenBenchmarker('Node.js', config.NODE_JS, ['-sMEMORY64=2']),
+    ]
+  else:
+    benchmarkers += [
+      # EmscriptenBenchmarker('Node.js', config.NODE_JS),
+    ]
 
 
 class benchmark(common.RunnerCore):
