@@ -29,8 +29,6 @@ class posixtest(RunnerCore):
 
 def filter_tests(all_tests):
   pthread_tests = [t for t in all_tests if t.startswith('pthread_')]
-  # filter out some tests we don't support
-  pthread_tests = [t for t in pthread_tests if not t.startswith('pthread_sigmask')]
   return pthread_tests
 
 
@@ -56,21 +54,22 @@ unsupported_noreturn = {
   'test_pthread_atfork_3_2': 'fork() and multiple processes are not supported',
   'test_pthread_atfork_4_1': 'fork() and multiple processes are not supported',
   'test_pthread_kill_1_1': 'signals are not supported',
-  'test_pthread_create_1_5': 'semaphores are not supported',
+  'test_pthread_create_1_5': 'fork() and multiple processes are not supported',
   'test_pthread_exit_6_1': 'lacking necessary mmap() support',
   'test_pthread_spin_lock_1_1': 'signals are not supported',
   'test_pthread_mutex_lock_5_1': 'signals are not supported',
   'test_pthread_mutexattr_settype_2_1': 'interrupting pthread_mutex_lock wait via SIGALRM is not supported',
   'test_pthread_spin_lock_3_1': 'signals are not supported',
-  'test_pthread_mutex_lock_3_1': 'signals are not supported',
-  'test_pthread_create_14_1': 'signals are not supported',
-  'test_pthread_detach_4_3': 'signals are not supported',
-  'test_pthread_join_6_3': 'signals are not supported',
-  'test_pthread_cond_init_4_2': 'signals are not supported',
+  'test_pthread_create_14_1': 'creates too many threads',
+  'test_pthread_detach_4_3': 'creates too many threads',
+  'test_pthread_join_6_3': 'creates too many threads',
   'test_pthread_barrier_wait_3_2': 'signals are not supported',
+  'test_pthread_cond_broadcast_1_2': 'tries to create 10,0000 threads, then depends on fork()',
 }
 
 unsupported = {
+  'test_pthread_attr_setinheritsched_2_2': 'scheduling policy/parameters are not supported',
+  'test_pthread_attr_setinheritsched_2_3': 'scheduling policy/parameters are not supported',
   'test_pthread_attr_setinheritsched_2_3': 'scheduling policy/parameters are not supported',
   'test_pthread_attr_setinheritsched_2_4': 'scheduling policy/parameters are not supported',
   'test_pthread_attr_setschedparam_1_3': 'scheduling policy/parameters are not supported',
@@ -78,7 +77,6 @@ unsupported = {
   'test_pthread_attr_setschedpolicy_4_1': 'scheduling policy/parameters are not supported',
   'test_pthread_barrierattr_getpshared_2_1': 'shm_open and shm_unlink are not supported',
   'test_pthread_barrier_wait_3_1': 'signals are not supported',
-  'test_pthread_cond_broadcast_1_2': 'lacking necessary mmap() support',
   'test_pthread_cond_broadcast_2_3': 'lacking necessary mmap() support',
   'test_pthread_cond_destroy_2_1': 'lacking necessary mmap() support',
   'test_pthread_cond_init_1_2': 'clock_settime() is not supported',
@@ -92,8 +90,10 @@ unsupported = {
   'test_pthread_cond_wait_2_2': 'lacking necessary mmap() support',
   'test_pthread_create_8_1': 'signals are not supported',
   'test_pthread_create_8_2': 'signals are not supported',
-  'test_pthread_create_10_1': 'signals are not supported',
+  'test_pthread_create_11_1': '_POSIX_THREAD_CPUTIME not supported',
+  'test_pthread_getcpuclockid_1_1': 'pthread_getcpuclockid not supported',
   'test_pthread_getschedparam_1_3': 'scheduling policy/parameters are not supported',
+  'test_pthread_getschedparam_1_2': 'scheduling policy/parameters are not supported',
   'test_pthread_kill_1_2': 'signals are not supported',
   'test_pthread_mutexattr_getprioceiling_1_2': 'pthread_mutexattr_setprioceiling is not supported',
   'test_pthread_mutexattr_getprotocol_1_2': 'pthread_mutexattr_setprotocol is not supported',
@@ -115,6 +115,10 @@ unsupported = {
   'test_pthread_rwlock_timedwrlock_6_1': 'signals are not supported',
   'test_pthread_rwlock_timedwrlock_6_2': 'signals are not supported',
   'test_pthread_rwlock_wrlock_2_1': 'signals are not supported',
+  'test_pthread_setschedparam_1_1': 'scheduling policy/parameters are not supported',
+  'test_pthread_setschedparam_1_2': 'scheduling policy/parameters are not supported',
+  'test_pthread_setschedparam_4_1': 'scheduling policy/parameters are not supported',
+  'test_pthread_setschedprio_1_1': 'scheduling policy/parameters are not supported',
   'test_pthread_spin_init_2_1': 'shm_open and shm_unlink are not supported',
   'test_pthread_spin_init_2_2': 'shm_open and shm_unlink are not supported',
 }
@@ -125,6 +129,8 @@ flaky = {
   'test_pthread_cond_signal_1_1': 'flaky: https://github.com/emscripten-core/emscripten/issues/13283',
   'test_pthread_barrier_wait_2_1': 'flaky: https://github.com/emscripten-core/emscripten/issues/14508',
   'test_pthread_rwlock_unlock_3_1': 'Test fail: writer did not get write lock, when main release the lock',
+  'test_pthread_mutex_init_1_2': 'flaky: https://github.com/emscripten-core/posixtestsuite/pull/9',
+  'test_pthread_mutex_init_3_2': 'flaky: https://github.com/emscripten-core/posixtestsuite/pull/9',
 }
 
 # Mark certain tests as disabled.  These are tests that are either flaky or never return.
@@ -158,7 +164,7 @@ def make_test(name, testfile, browser):
     if browser:
       self.btest_exit(testfile, args=args)
     else:
-      self.do_runf(testfile, emcc_args=args)
+      self.do_runf(testfile, emcc_args=args, output_basename=name)
 
   if name in expect_fail:
     f = unittest.expectedFailure(f)
