@@ -294,117 +294,6 @@ class sockets(BrowserCore):
       with harness:
         self.btest(os.path.join('sockets', 'test_enet_client.c'), expected='0', args=enet + ['-DSOCKK=%d' % harness.listen_port])
 
-  # This test is no longer in use for WebSockets as we can't truly emulate
-  # a server in the browser (in the past, there were some hacks to make it
-  # somewhat work, but those have been removed). However, with WebRTC it
-  # should be able to resurect this test.
-  # def test_enet_in_browser(self):
-  #   shared.try_delete('enet')
-  #   shutil.copytree(test_file('enet'), 'enet')
-  #   pwd = os.getcwd()
-  #   os.chdir('enet')
-  #   self.run_process([path_from_root('emconfigure'), './configure'])
-  #   self.run_process([path_from_root('emmake'), 'make'])
-  #   enet = [self.in_dir('enet', '.libs', 'libenet.a'), '-I' + test_file('enet', 'include')]
-  #   os.chdir(pwd)
-  #   self.run_process([EMCC, test_file('sockets', 'test_enet_server.c'), '-o', 'server.html', '-DSOCKK=2235'] + enet)
-  #   def make_relay_server(port1, port2):
-  #     print('creating relay server on ports %d,%d' % (port1, port2), file=sys.stderr)
-  #     proc = self.run_process([PYTHON, test_file('sockets', 'socket_relay.py'), str(port1), str(port2)])
-  #     return proc
-  #   with WebsockifyServerHarness('', [], 2235, 2234):
-  #     with WebsockifyServerHarness('', [], 2237, 2236):
-  #       pids = []
-  #       try:
-  #         proc = make_relay_server(2234, 2236)
-  #         pids.append(proc.pid)
-  #         self.btest(os.path.join('sockets', 'test_enet_client.c'), expected='0', args=['-DSOCKK=2237', '-DUSE_IFRAME=1'] + enet)
-  #       finally:
-  #         clean_pids(pids);
-
-  def test_webrtc(self): # XXX see src/settings.js, this is disabled pending investigation
-    self.skipTest('WebRTC support is not up to date.')
-    host_src = 'webrtc_host.c'
-    peer_src = 'webrtc_peer.c'
-
-    host_outfile = 'host.html'
-    peer_outfile = 'peer.html'
-
-    host_filepath = test_file('sockets', host_src)
-    temp_host_filepath = os.path.join(self.get_dir(), os.path.basename(host_src))
-    host_src = read_file(host_filepath)
-    with open(temp_host_filepath, 'w') as f:
-      f.write(host_src)
-
-    peer_filepath = test_file('sockets', peer_src)
-    temp_peer_filepath = os.path.join(self.get_dir(), os.path.basename(peer_src))
-    peer_src = read_file(peer_filepath)
-    with open(temp_peer_filepath, 'w') as f:
-      f.write(peer_src)
-
-    create_file('host_pre.js', '''
-      var Module = {
-        webrtc: {
-          broker: 'http://localhost:8182',
-          session: undefined,
-          onpeer: function(peer, route) {
-            window.open('http://localhost:8888/peer.html?' + route);
-            // iframe = document.createElement("IFRAME");
-            // iframe.setAttribute("src", "http://localhost:8888/peer.html?" + route);
-            // iframe.style.display = "none";
-            // document.body.appendChild(iframe);
-            peer.listen();
-          },
-          onconnect: function(peer) {
-          },
-          ondisconnect: function(peer) {
-          },
-          onerror: function(error) {
-            console.error(error);
-          }
-        },
-        setStatus: function(text) {
-          console.log('status: ' + text);
-        }
-      };
-    ''')
-
-    create_file('peer_pre.js', '''
-      var Module = {
-        webrtc: {
-          broker: 'http://localhost:8182',
-          session: window.location.toString().split('?')[1],
-          onpeer: function(peer, route) {
-            peer.connect(Module['webrtc']['session']);
-          },
-          onconnect: function(peer) {
-          },
-          ondisconnect: function(peer) {
-            // Calling window.close() from this handler hangs my browser, so run it in the next turn
-            setTimeout(window.close, 0);
-          },
-          onerror: function(error) {
-            console.error(error);
-          },
-        },
-        setStatus: function(text) {
-          console.log('status: ' + text);
-        }
-      };
-    ''')
-
-    self.compile_btest(['-Werror', temp_host_filepath, '-o', host_outfile] + ['-s', 'GL_TESTING', '--pre-js', 'host_pre.js', '-s', 'SOCKET_WEBRTC', '-s', 'SOCKET_DEBUG'])
-    self.compile_btest(['-Werror', temp_peer_filepath, '-o', peer_outfile] + ['-s', 'GL_TESTING', '--pre-js', 'peer_pre.js', '-s', 'SOCKET_WEBRTC', '-s', 'SOCKET_DEBUG'])
-
-    # note: you may need to run this manually yourself, if npm is not in the path, or if you need a version that is not in the path
-    self.run_process([NPM, 'install', test_file('sockets', 'p2p')])
-    broker = Popen(config.NODE_JS + [test_file('sockets', 'p2p', 'broker', 'p2p-broker.js')])
-
-    expected = '1'
-    self.run_browser(host_outfile, '.', ['/report_result?' + e for e in expected])
-
-    broker.kill()
-
   def test_nodejs_sockets_echo(self):
     # This test checks that sockets work when the client code is run in Node.js
     if config.NODE_JS not in config.JS_ENGINES:
@@ -475,7 +364,7 @@ class sockets(BrowserCore):
   # Test that native POSIX sockets API can be used by proxying calls to an intermediate WebSockets -> POSIX sockets bridge server
   def test_posix_proxy_sockets(self):
     # Build the websocket bridge server
-    self.run_process(['cmake', path_from_root('tools', 'websocket_to_posix_proxy')])
+    self.run_process(['cmake', path_from_root('tools/websocket_to_posix_proxy')])
     self.run_process(['cmake', '--build', '.'])
     if os.name == 'nt': # This is not quite exact, instead of "isWindows()" this should be "If CMake defaults to building with Visual Studio", but there is no good check for that, so assume Windows==VS.
       proxy_server = os.path.join(self.get_dir(), 'Debug', 'websocket_to_posix_proxy.exe')
