@@ -15,8 +15,8 @@ import os
 import re
 import shutil
 import subprocess
-import time
 import signal
+import stat
 import sys
 import tempfile
 
@@ -25,9 +25,19 @@ if sys.version_info < (3, 6):
   print('error: emscripten requires python 3.6 or above', file=sys.stderr)
   sys.exit(1)
 
+from . import colored_logger
+
+# Configure logging before importing any other local modules so even
+# log message during import are shown as expected.
+DEBUG = int(os.environ.get('EMCC_DEBUG', '0'))
+# can add  %(asctime)s  to see timestamps
+logging.basicConfig(format='%(name)s:%(levelname)s: %(message)s',
+                    level=logging.DEBUG if DEBUG else logging.INFO)
+colored_logger.enable()
+
 from .tempfiles import try_delete
 from .utils import path_from_root, exit_with_error, safe_ensure_dirs, WINDOWS
-from . import cache, tempfiles, colored_logger
+from . import cache, tempfiles
 from . import diagnostics
 from . import config
 from . import filelock
@@ -35,19 +45,13 @@ from . import utils
 from .settings import settings
 
 
-DEBUG = int(os.environ.get('EMCC_DEBUG', '0'))
 DEBUG_SAVE = DEBUG or int(os.environ.get('EMCC_DEBUG_SAVE', '0'))
 EXPECTED_NODE_VERSION = (4, 1, 1)
-EXPECTED_LLVM_VERSION = "13.0"
+EXPECTED_LLVM_VERSION = "14.0"
 PYTHON = sys.executable
 
 # Used only when EM_PYTHON_MULTIPROCESSING=1 env. var is set.
 multiprocessing_pool = None
-
-# can add  %(asctime)s  to see timestamps
-logging.basicConfig(format='%(name)s:%(levelname)s: %(message)s',
-                    level=logging.DEBUG if DEBUG else logging.INFO)
-colored_logger.enable()
 logger = logging.getLogger('shared')
 
 # warning about absolute-paths is disabled by default, and not enabled by -Wall
@@ -220,40 +224,21 @@ def check_call(cmd, *args, **kw):
     exit_with_error("'%s' failed: %s", shlex_join(cmd), str(e))
 
 
-def run_js_tool(filename, jsargs=[], *args, **kw):
+def run_js_tool(filename, jsargs=[], node_args=[], **kw):
   """Execute a javascript tool.
 
   This is used by emcc to run parts of the build process that are written
   implemented in javascript.
   """
-  command = config.NODE_JS + [filename] + jsargs
-  return check_call(command, *args, **kw).stdout
-
-
-# Only used by tests and by ctor_evaller.py.   Once fastcomp is removed
-# this can most likely be moved into the tests/jsrun.py.
-def timeout_run(proc, timeout=None, full_output=False, check=True):
-  start = time.time()
-  if timeout is not None:
-    while time.time() - start < timeout and proc.poll() is None:
-      time.sleep(0.1)
-    if proc.poll() is None:
-      proc.kill() # XXX bug: killing emscripten.py does not kill it's child process!
-      raise Exception("Timed out")
-  stdout, stderr = proc.communicate()
-  out = ['' if o is None else o for o in (stdout, stderr)]
-  if check and proc.returncode != 0:
-    raise subprocess.CalledProcessError(proc.returncode, '', stdout, stderr)
-  if TRACK_PROCESS_SPAWNS:
-    logging.info(f'Process {proc.pid} finished after {time.time() - start} seconds. Exit code: {proc.returncode}')
-  return '\n'.join(out) if full_output else out[0]
+  command = config.NODE_JS + node_args + [filename] + jsargs
+  return check_call(command, **kw).stdout
 
 
 def get_npm_cmd(name):
   if WINDOWS:
-    cmd = [path_from_root('node_modules', '.bin', name + '.cmd')]
+    cmd = [path_from_root('node_modules/.bin', name + '.cmd')]
   else:
-    cmd = config.NODE_JS + [path_from_root('node_modules', '.bin', name)]
+    cmd = config.NODE_JS + [path_from_root('node_modules/.bin', name)]
   if not os.path.exists(cmd[-1]):
     exit_with_error(f'{name} was not found! Please run "npm install" in Emscripten root directory to set up npm dependencies')
   return cmd
@@ -334,9 +319,8 @@ def check_node_version():
 def set_version_globals():
   global EMSCRIPTEN_VERSION, EMSCRIPTEN_VERSION_MAJOR, EMSCRIPTEN_VERSION_MINOR, EMSCRIPTEN_VERSION_TINY
   filename = path_from_root('emscripten-version.txt')
-  with open(filename) as f:
-    EMSCRIPTEN_VERSION = f.read().strip().strip('"')
-  parts = [int(x) for x in EMSCRIPTEN_VERSION.split('.')]
+  EMSCRIPTEN_VERSION = utils.read_file(filename).strip().strip('"')
+  parts = [int(x) for x in EMSCRIPTEN_VERSION.split('-')[0].split('.')]
   EMSCRIPTEN_VERSION_MAJOR, EMSCRIPTEN_VERSION_MINOR, EMSCRIPTEN_VERSION_TINY = parts
 
 
@@ -432,8 +416,7 @@ def check_sanity(force=False):
 
     if not force:
       # Only create/update this file if the sanity check succeeded, i.e., we got here
-      with open(sanity_file, 'w') as f:
-        f.write(expected)
+      utils.write_file(sanity_file, expected)
 
 
 # Some distributions ship with multiple llvm versions so they add
@@ -620,15 +603,13 @@ class JS:
   @staticmethod
   def handle_license(js_target):
     # ensure we emit the license if and only if we need to, and exactly once
-    with open(js_target) as f:
-      js = f.read()
+    js = utils.read_file(js_target)
     # first, remove the license as there may be more than once
     processed_js = re.sub(JS.emscripten_license_regex, '', js)
     if settings.EMIT_EMSCRIPTEN_LICENSE:
       processed_js = JS.emscripten_license + processed_js
     if processed_js != js:
-      with open(js_target, 'w') as f:
-        f.write(processed_js)
+      utils.write_file(js_target, processed_js)
 
   @staticmethod
   def to_nice_ident(ident): # limited version of the JS function toNiceIdent
@@ -649,8 +630,7 @@ class JS:
       # if the path does not exist, then there is no data to encode
       if not os.path.exists(path):
         return ''
-      with open(path, 'rb') as f:
-        data = base64.b64encode(f.read())
+      data = base64.b64encode(utils.read_binary(path))
       return 'data:application/octet-stream;base64,' + data.decode('ascii')
     else:
       return os.path.basename(path)
@@ -698,7 +678,7 @@ class JS:
       else:
         return 'Module["dynCall_%s"](%s)' % (sig, args)
     else:
-      return 'wasmTable.get(%s)(%s)' % (args[0], ','.join(args[1:]))
+      return 'getWasmTableEntry(%s)(%s)' % (args[0], ','.join(args[1:]))
 
   @staticmethod
   def make_invoke(sig, named=True):
@@ -750,6 +730,12 @@ def strip_prefix(string, prefix):
   return string[len(prefix):]
 
 
+def make_writable(filename):
+  assert(os.path.isfile(filename))
+  old_mode = stat.S_IMODE(os.stat(filename).st_mode)
+  os.chmod(filename, old_mode | stat.S_IWUSR)
+
+
 def safe_copy(src, dst):
   logging.debug('copy: %s -> %s', src, dst)
   src = os.path.abspath(src)
@@ -762,6 +748,9 @@ def safe_copy(src, dst):
     return
   # Copies data and permission bits, but not other metadata such as timestamp
   shutil.copy(src, dst)
+  # We always want the target file to be writable even when copying from
+  # read-only source. (e.g. a read-only install of emscripten).
+  make_writable(dst)
 
 
 def read_and_preprocess(filename, expand_macros=False):
@@ -775,8 +764,7 @@ def read_and_preprocess(filename, expand_macros=False):
     settings_str += f'var {key} = {jsoned};\n'
 
   settings_file = os.path.join(temp_dir, 'settings.js')
-  with open(settings_file, 'w') as f:
-    f.write(settings_str)
+  utils.write_file(settings_file, settings_str)
 
   # Run the JS preprocessor
   # N.B. We can't use the default stdout=PIPE here as it only allows 64K of output before it hangs
@@ -790,7 +778,7 @@ def read_and_preprocess(filename, expand_macros=False):
   if expand_macros:
     args += ['--expandMacros']
 
-  run_js_tool(path_from_root('tools/preprocessor.js'), args, True, stdout=open(stdout, 'w'), cwd=dirname)
+  run_js_tool(path_from_root('tools/preprocessor.js'), args, stdout=open(stdout, 'w'), cwd=dirname)
   out = utils.read_file(stdout)
 
   return out
@@ -841,7 +829,7 @@ EMRANLIB = bat_suffix(path_from_root('emranlib'))
 EMCMAKE = bat_suffix(path_from_root('emcmake'))
 EMCONFIGURE = bat_suffix(path_from_root('emconfigure'))
 EM_NM = bat_suffix(path_from_root('emnm'))
-FILE_PACKAGER = bat_suffix(path_from_root('tools', 'file_packager'))
+FILE_PACKAGER = bat_suffix(path_from_root('tools/file_packager'))
 
 apply_configuration()
 
